@@ -4,9 +4,9 @@
 
 import logging
 from collections import defaultdict
-from typing import TYPE_CHECKING, Optional, Collection, Any
+from typing import TYPE_CHECKING, Optional, Collection
 
-from .context import Context
+from .context import ctx
 from .exceptions import CommandDefinitionError, ParameterDefinitionError
 from .formatting import HelpFormatter
 from .parameters import (
@@ -22,7 +22,7 @@ from .parameters import (
 )
 
 if TYPE_CHECKING:
-    from .commands import CommandType
+    from .core import CommandType
 
 __all__ = ['CommandParameters']
 log = logging.getLogger(__name__)
@@ -46,7 +46,7 @@ class CommandParameters:
         self.command = command
         if command_parent:
             self.command_parent = command_parent
-            self.parent = command_parent.params
+            self.parent = command_parent.__class__.params(command_parent)
 
         self.formatter = HelpFormatter(command, self)
         self._process_parameters()
@@ -253,7 +253,7 @@ class CommandParameters:
         key, value = option[1], option[2:]
         # value will never be empty if key is a valid option because by this point, option is not a short option
         param = self.combo_option_map[key]
-        if param.would_accept(Context.get_current(), value, short_combo=True):
+        if param.would_accept(value, short_combo=True):
             return [(param, value)]
         else:
             combo_option_map = self.combo_option_map
@@ -275,7 +275,8 @@ class CommandParameters:
     def find_nested_option_that_accepts_values(self, option: str):
         if sub_command := self.sub_command:
             for choice in sub_command.choices.values():
-                params = choice.target.params
+                command = choice.target
+                params = command.__class__.params(command)
                 try:
                     param = params.find_option_that_accepts_values(option)
                 except KeyError:
@@ -289,3 +290,12 @@ class CommandParameters:
         return None
 
     # endregion
+
+    def missing(self) -> list['Parameter']:
+        # ignore = (SubCommand, Action)
+        ignore = SubCommand
+        missing: list['Parameter'] = [
+            p for p in self.positionals if p.required and ctx.num_provided(p) == 0 and not isinstance(p, ignore)
+        ]
+        missing.extend(p for p in self.options if p.required and ctx.num_provided(p) == 0)
+        return missing
