@@ -4,21 +4,29 @@
 
 import sys
 from collections import defaultdict
+from contextlib import AbstractContextManager
+from contextvars import ContextVar
 from functools import cached_property
-from threading import local
-from typing import TYPE_CHECKING, Any, Union, Sequence, Optional, Type, Iterator, Collection
+from typing import TYPE_CHECKING, Any, Union, Sequence, Optional, Iterator, Collection, cast
 
-from .parameters import Parameter, ParamOrGroup, Param, ActionFlag, SubCommand, Action
-from .utils import Bool
+from .error_handling import ErrorHandler, NullErrorHandler, extended_error_handler
+from .exceptions import NoActiveContext
+from .utils import Bool, _NotSet
 
 if TYPE_CHECKING:
-    from .commands import CommandType
+    from .core import CommandType
     from .command_parameters import CommandParameters
+    from .parameters import Parameter, ParamOrGroup, ActionFlag
 
-__all__ = ['Context']
+__all__ = ['Context', 'ctx', 'get_current_context']
+
+_context_stack = ContextVar('cli_command_parser.context.stack', default=[])
 
 
 class ConfigOption:
+    def __init__(self, default: Any = None):
+        self.default = default
+
     def __set_name__(self, owner, name: str):
         self.name = name
 
@@ -38,25 +46,37 @@ class ConfigOption:
             return self.get_value(ctx, ctx_cls)
         except KeyError:
             pass
+
+        command = ctx.command
         try:
-            config = ctx.command._config_
+            config = command.__class__.config(command)
         except AttributeError:
-            return None
+            return self.default
         else:
             return getattr(config, self.name)
 
     def __set__(self, ctx: 'Context', value: Optional[Bool]):
-        if value is not None:
+        if value is not self.default:
             ctx.__dict__[self.name] = value
 
 
-class Context:
-    _local = local()
+class Context(AbstractContextManager):  # Extending AbstractContextManager to make PyCharm's type checker happy
+    """
+    The parsing context.
+
+    Holds user input while parsing, and holds the parsed values.  Handles config overrides / hierarchy for settings that
+    affect parser behavior.
+    """
+
+    error_handler = ConfigOption(_NotSet)
     ignore_unknown = ConfigOption()
     # parse_unknown = ConfigOption()
     allow_missing = ConfigOption()
     multiple_action_flags = ConfigOption()
     action_after_action_flags = ConfigOption()
+    # strict_option_punctuation = ConfigOption()
+    # strict_action_punctuation = ConfigOption()
+    # strict_sub_command_punctuation = ConfigOption()
 
     def __init__(
         self,
@@ -64,11 +84,15 @@ class Context:
         command: Optional['CommandType'] = None,
         parent: Optional['Context'] = None,
         *,
+        error_handler: Optional[ErrorHandler] = _NotSet,
         ignore_unknown: Bool = None,
         # parse_unknown: Bool = None,
         allow_missing: Bool = None,
         multiple_action_flags: Bool = None,
         action_after_action_flags: Bool = None,
+        # strict_option_punctuation: Bool = None,
+        # strict_action_punctuation: Bool = None,
+        # strict_sub_command_punctuation: Bool = None,
     ):
         self.argv = sys.argv[1:] if argv is None else argv
         self.remaining = list(self.argv)
@@ -84,57 +108,67 @@ class Context:
             self._provided = defaultdict(int)
         self.actions_taken = 0
         # Command config overrides
+        self.error_handler = error_handler
         self.ignore_unknown = ignore_unknown
         # self.parse_unknown = parse_unknown
         self.allow_missing = allow_missing
         self.multiple_action_flags = multiple_action_flags
         self.action_after_action_flags = action_after_action_flags
-
-    @classmethod
-    def get_current(cls, silent: bool = False) -> Optional['Context']:
-        try:
-            return cls._local.stack[-1]
-        except (AttributeError, IndexError) as e:
-            if silent:
-                return None
-            raise RuntimeError('There is no active context') from e
+        # self.strict_option_punctuation = strict_option_punctuation
+        # self.strict_action_punctuation = strict_action_punctuation
+        # self.strict_sub_command_punctuation = strict_sub_command_punctuation
 
     def __enter__(self) -> 'Context':
-        self._local.__dict__.setdefault('stack', []).append(self)
+        _context_stack.get().append(self)
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self._local.stack.pop()
+        _context_stack.get().pop()
 
     @cached_property
     def params(self) -> Optional['CommandParameters']:
         try:
-            return self.command.params
+            return self.command.__class__.params(self.command)
         except AttributeError:  # self.command is None
             return None
 
-    def get_parsed(self, exclude: Collection[Parameter] = (), recursive: Bool = True) -> dict[str, Any]:
-        if recursive and (parent := self.parent):
-            parsed = parent.get_parsed(exclude, recursive)
+    def get_error_handler(self) -> Union[ErrorHandler, NullErrorHandler]:
+        error_handler = self.error_handler
+        if error_handler is _NotSet:
+            return extended_error_handler
+        elif error_handler is None:
+            return NullErrorHandler()
         else:
-            parsed = {}
+            return error_handler
 
-        if params := self.params:
-            for group in (params.positionals, params.options, (params.pass_thru,)):
-                for param in group:
-                    if param and param not in exclude:
-                        parsed[param.name] = param.result_value(self)
+    def _sub_context(self, command: 'CommandType', argv: Optional[Sequence[str]] = None) -> 'Context':
+        if argv is None:
+            argv = self.remaining
+        return self.__class__(argv, command, parent=self)
+
+    def get_parsed(self, exclude: Collection['Parameter'] = (), recursive: Bool = True) -> dict[str, Any]:
+        with self:
+            if recursive and (parent := self.parent):
+                parsed = parent.get_parsed(exclude, recursive)
+            else:
+                parsed = {}
+
+            if params := self.params:
+                for group in (params.positionals, params.options, (params.pass_thru,)):
+                    for param in group:
+                        if param and param not in exclude:
+                            parsed[param.name] = param.result_value()
 
         return parsed
 
-    def get_parsing_value(self, param: Parameter):
+    def get_parsing_value(self, param: 'Parameter'):
         try:
             return self._parsing[param]
         except KeyError:
             self._parsing[param] = value = param._init_value_factory()
             return value
 
-    def set_parsing_value(self, param: Parameter, value: Any):
+    def set_parsing_value(self, param: 'Parameter', value: Any):
         self._parsing[param] = value
 
     def __contains__(self, param: Union['ParamOrGroup', str, Any]) -> bool:
@@ -158,16 +192,6 @@ class Context:
     def num_provided(self, param: 'ParamOrGroup') -> int:
         return self._provided[param]
 
-    def missing(self) -> list[Parameter]:
-        params = self.params
-        # ignore = (SubCommand, Action)
-        ignore = SubCommand
-        missing: list[Parameter] = [
-            p for p in params.positionals if p.required and self.num_provided(p) == 0 and not isinstance(p, ignore)
-        ]
-        missing.extend(p for p in params.options if p.required and self.num_provided(p) == 0)
-        return missing
-
     @cached_property
     def parsed_action_flags(self) -> tuple[int, list['ActionFlag'], list['ActionFlag']]:
         parsing = self._parsing
@@ -195,3 +219,48 @@ class Context:
         for action_flag in self.parsed_action_flags[2]:
             self.actions_taken += 1
             yield action_flag
+
+
+def get_current_context(silent: bool = False) -> Optional[Context]:
+    """
+    Get the currently active parsing context.
+
+    :param silent: If True, allow this function to return ``None`` if there is no active :class:`Context`
+    :return: The active :class:`Context` object
+    :raises: :class:`~.exceptions.NoActiveContext` if there is no active Context and ``silent=False`` (default)
+    """
+    try:
+        return _context_stack.get()[-1]
+    except (AttributeError, IndexError):
+        if silent:
+            return None
+        raise NoActiveContext('There is no active context') from None
+
+
+class ContextProxy:
+    """
+    Proxy for the currently active :class:`Context` object.  Allows usage similar to the ``request`` object in Flask.
+
+    This class should not be instantiated by users - use the common :data:`ctx` instance.
+    """
+
+    def __getattr__(self, attr: str):
+        return getattr(get_current_context(), attr)
+
+    def __setattr__(self, attr: str, value):
+        return setattr(get_current_context(), attr, value)
+
+    def __eq__(self, other) -> bool:
+        return get_current_context() == other
+
+    def __contains__(self, item) -> bool:
+        return item in get_current_context()
+
+    def __enter__(self):
+        return get_current_context().__enter__()
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return get_current_context().__exit__(exc_type, exc_val, exc_tb)
+
+
+ctx: Context = cast(Context, ContextProxy())
